# mockextras.stub
# Matchers and Stubs for mock.
# Copyright (C) 2012-2015 Man AHL
# E-mail: ManAHLTech AT ahl DOT com

# mockextras 1.0.0
# https://github.com/manahl/mockextras

# Released subject to the BSD License
# Please see https://github.com/manahl/mockextras/blob/master/LICENSE.txt

from ._matchers import __all__ as matchers_all
try:
    from unittest.mock import _is_exception, call
except ImportError:
    try:
        from mock.mock import call, _is_exception 
    except ImportError:
        from mock import call, _is_exception
from os import linesep


__all__ = ['seq', 'stub', 'UnexpectedStubCall']


class _Sequence(object):
    def __init__(self, iterable):
        self._iterator = iter(iterable)

    def __call__(self):
        retval = next(self._iterator)
        if _is_exception(retval):
            raise retval
        return retval


def seq(iterable):
    """Used to define a sequence of return values for a stub based on an iterable, such as a container:
    
    >>> try:
    ...     from unittest.mock Mock, call
    ... except ImportError:
    ...     from mock import Mock, call
    >>>
    >>> l = range(1, 5)
    >>> fn = stub((call(), seq(l)))
    >>> fn()
    1
    >>> fn()
    2
    >>> fn()
    3

    or a generator:

    >>> i = xrange(1, 5)
    >>> fn = stub((call(), seq(i)))
    >>> fn()
    1
    >>> fn()
    2
    >>> fn()
    3
    """
    return _Sequence(iterable)


class UnexpectedStubCall(Exception):
    pass


def _one_per_line_indented(results, indent=4):
    return ("""
""" + " " * indent).join(str(k) for k, _ in results)


class _Stub(object):
    def __init__(self, *args):
        self._results = tuple((conf[0], seq(conf[1:])) if len(conf) > 2 else conf for conf in args)

    def _lookup(self, k):
        for key, value in self._results:
            # Some classes don't play by the rules so try the equals both ways around
            if key == k or k == key:
                return value
        if self._results:
            raise UnexpectedStubCall("""Unexpected stub call:
    %s
The following calls are configured:
    %s
""" % (k, _one_per_line_indented(self._results)))
        else:
            raise UnexpectedStubCall("Unexpected call of an unconfigured stub")

    def __call__(self, *args, **kwargs):
        obj = self._lookup(call(*args, **kwargs))
        if _is_exception(obj):
            raise obj
        if isinstance(obj, _Sequence):
            return obj()
        return obj


def stub(*args):
    return _Stub(*args)


stub.__doc__ = """Makes stubs that can be used stand-alone or with mock.

Stubs are dumb functions, used in testing, they do no processing but they can take arguments and return 
predefined results.

A stub is configured so it returns different values depending on the arguments passed to it. You configure
it with one or more pairs of call arguments and results then when the stub is called with a given set of call
arguments the corresponding result is returned. If the result is an Exception the result is raised. If more
than one result is specified the results will be returned/raised one at a time over successive calls to the
stub. If you wish to specify successive results using an iterable you must wrap it with seq().

You can use a stub in place of a function, for example:

>>> try:
...     from unittest.mock call
... except ImportError:
...     from mock import call
>>>
>>> fn = stub((call("hello"), "world"),
...           (call("foo"),   1, 2, 4, 8),
...           (call("bar"),   seq(xrange(100))),
...           (call("baz"),   KeyError('baz')),
...           (call("boom"),  100, RuntimeError, 200, ValueError("boom")))
>>> fn("hello")
'world'
>>> fn("foo")
1
>>> fn("foo")
2
>>> fn("foo")
4

Or you can combine it with a mock by setting it as the side_effect. This has the advantage that you can later
verify the function was called as expected.

>>> try:
...     from unittest.mock Mock, call
... except ImportError:
...     from mock import Mock, call
>>>
>>> mock = Mock()
>>> mock.side_effect = stub((call("hello"), "world"),
...                         (call("foo"),   1,2,4,8))
>>> mock("hello")
'world'
>>> mock("foo")
1
>>> mock("foo")
2
>>> mock("foo")
4
>>> assert mock.call_args_list == [call("hello"), call("foo"), call("foo"), call("foo")]

Also you can use stubs as methods on Mock objects. Whether you use them directly as the methods or as the
side_effect of a mock method depends on whether you want to verify the method calls.

>>> mock_obj = Mock(my_first_method=stub((call(50), 100), (call(100), 200)))
>>> mock_obj.my_second_method = stub((call("a"), "aa"), (call("b"), "bb"))
>>> mock_obj.my_third_method.side_effect = stub((call(123), 456), (call(789), 54321))

>>> mock_obj.my_first_method(50)
100
>>> mock_obj.my_second_method('b')
'bb'
>>> mock_obj.my_third_method(123)
456
>>> assert mock_obj.mock_calls == [call.my_third_method(123)] # only the mocked call is recorded

You can use matchers, such as Any(), as wild-card arguments when matching call arguments. The stub's
configuration is searched in the order it was specified so you can put more specific call argument
specifications ahead of more general ones.

For example:

>>> from mockextras import Any
>>> fn = stub((call(100, 200),   "monkey"),
...           (call(100, Any()), "hello"))
>>> fn(100, 200)
'monkey'
>>> fn(100, 300)
'hello'
>>> fn(100, "monkey")
'hello'
>>> fn(100, { "key" : 1000 })
'hello'

The following matchers are available in mockextras:
%s

See their documentation for more info.
""" % linesep.join('  * %s' % m for m in matchers_all)



from mockextras import Any, Contains, AnyOf


def test_any_equality():
    assert Any() == "hello"
    assert "hello" == Any()

    assert Any() == []
    assert [] == Any()

    assert Any() == 100
    assert 100 == Any()

    assert Any() == []
    assert [] == Any()

    assert Any() == 100
    assert 100 == Any()

    assert Any(str) == "hello"
    assert "hello" == Any(str)

    assert Any(list) != "hello"
    assert "hello" != Any(list)

    assert Any(list) == []
    assert [] == Any(list)


def test_pretty_print_any():
    a = Any()
    assert repr(a) == "Any()"
    assert str(a) == "Any()"
    b = Any(str)
    assert repr(b) == "Any(<type 'str'>)" or repr(b) == "Any(<class 'str'>)"
    assert str(b) == "Any(<type 'str'>)" or str(b) == "Any(<class 'str'>)"


def test_contains_equality():
    assert Contains('h') == "hello"
    assert Contains('hell') == "hello"
    assert Contains('hello world') != "hello"
    assert Contains('fish') != "hello"

    assert "hello" == Contains('h')
    assert "hello" == Contains('hell')
    assert "hello" != Contains('hello world')
    assert "hello" != Contains('fish')

    assert range(100) == Contains(25)
    assert range(100) != Contains(125)

    assert dict(a=1, b=2, c=3) == Contains('a')
    assert dict(a=1, b=2, c=3) != Contains('d')
    assert Contains('a') == dict(a=1, b=2, c=3)
    assert Contains('d') != dict(a=1, b=2, c=3)


def test_pretty_print_contains():
    a = Contains(10)
    assert repr(a) == "Contains(10)"
    assert str(a) == "Contains(10)"

    b = Contains('hello')
    assert repr(b) == "Contains('hello')"
    assert str(b) == "Contains('hello')"


def test_any_of_equality():
    assert AnyOf() != "hello"
    assert AnyOf("fox") != "hello"
    assert AnyOf("fox", "badger", "monkey") != "hello"
    assert AnyOf('hello') == "hello"
    assert AnyOf('hello', 'world', 'hoorah') == "hello"

    assert "hello" != AnyOf()
    assert "hello" != AnyOf("fox")
    assert "hello" != AnyOf("fox", "badger", "monkey")
    assert "hello" == AnyOf('hello')
    assert "hello" == AnyOf('hello', 'world', 'hoorah')


def test_pretty_print_any_of():
    a = AnyOf()
    assert repr(a) == "AnyOf()"
    assert str(a) == "AnyOf()"

    a = AnyOf(10)
    assert repr(a) == "AnyOf(10)"
    assert str(a) == "AnyOf(10)"

    b = AnyOf(10, 20, 30)
    assert repr(b) == "AnyOf(10, 20, 30)"
    assert str(b) == "AnyOf(10, 20, 30)"



from mockextras import when, Any, UnexpectedStubCall
try:
    from unittest.mock import Mock, MagicMock, sentinel
except ImportError:
    from mock import Mock, MagicMock, sentinel
import pytest
import sys


def test_when_with_mock():
    mock_fn = Mock()
    when(mock_fn).called_with(sentinel.arg).then(sentinel.result)

    assert mock_fn(sentinel.arg) == sentinel.result


def test_when_with_magic_mock():
    mock_fn = MagicMock()
    when(mock_fn).called_with(sentinel.arg).then(sentinel.result)

    assert mock_fn(sentinel.arg) == sentinel.result


def test_can_not_use_when_with_non_mock():
    mock_fn = lambda _ : 10

    with pytest.raises(RuntimeError):
        when(mock_fn)


def test_can_not_use_when_with_mock_that_has_already_got_a_side_effect():
    with pytest.raises(RuntimeError):
        when(Mock(side_effect=lambda _ : 10))

    with pytest.raises(RuntimeError):
        when(Mock(side_effect=[1, 2, 3, 4]))


def test_when_with_any():
    mock_fn = Mock()
    when(mock_fn).called_with(Any()).then(sentinel.result1)
    when(mock_fn).called_with(sentinel.arg1, Any()).then(sentinel.result2)
    when(mock_fn).called_with(sentinel.arg2, Any(list)).then(sentinel.result3)
    when(mock_fn).called_with(sentinel.arg2, Any(str)).then(sentinel.result4)

    assert mock_fn(sentinel.arg1) == sentinel.result1
    assert mock_fn(sentinel.arg2) == sentinel.result1
    assert mock_fn("hello") == sentinel.result1
    assert mock_fn(100) == sentinel.result1
    assert mock_fn(sentinel.arg1, "hello") == sentinel.result2
    assert mock_fn(sentinel.arg1, "world") == sentinel.result2
    assert mock_fn(sentinel.arg2, []) == sentinel.result3
    assert mock_fn(sentinel.arg2, [1, 2, 3]) == sentinel.result3
    assert mock_fn(sentinel.arg2, ["hello", "world"]) == sentinel.result3
    assert mock_fn(sentinel.arg2, "world") == sentinel.result4


def test_when_call_then_return():
    mock_fn = Mock()
    when(mock_fn).called_with().then(sentinel.result0)
    when(mock_fn).called_with(sentinel.arg1).then(sentinel.result1)
    when(mock_fn).called_with(sentinel.arg2).then(sentinel.result2)
    when(mock_fn).called_with(sentinel.arg1, sentinel.arg2).then(sentinel.result3)
    when(mock_fn).called_with(sentinel.arg1, sentinel.arg1).then(sentinel.result4)
    when(mock_fn).called_with(sentinel.arg1, other=sentinel.other).then(sentinel.result5)
    when(mock_fn).called_with(x=sentinel.x, y=sentinel.y).then(sentinel.result6)

    assert mock_fn() == sentinel.result0
    assert mock_fn(sentinel.arg1) == sentinel.result1
    assert mock_fn(sentinel.arg2) == sentinel.result2
    assert mock_fn(sentinel.arg1, sentinel.arg2) == sentinel.result3
    assert mock_fn(sentinel.arg1, sentinel.arg1) == sentinel.result4
    assert mock_fn(sentinel.arg1, other=sentinel.other) == sentinel.result5
    assert mock_fn(x=sentinel.x, y=sentinel.y) == sentinel.result6


def test_when_call_then__return_single():
    mock_fn = Mock()
    when(mock_fn).called_with(sentinel.arg1).then(sentinel.result1)

    assert mock_fn(sentinel.arg1) == sentinel.result1
    assert mock_fn(sentinel.arg1) == sentinel.result1


def test_when_call_then_return_multiple():
    mock_fn = Mock()
    when(mock_fn).called_with(sentinel.arg1).then(sentinel.result1)\
                                            .then(sentinel.result2)\
                                            .then(sentinel.result3)

    assert mock_fn(sentinel.arg1) == sentinel.result1
    assert mock_fn(sentinel.arg1) == sentinel.result2
    assert mock_fn(sentinel.arg1) == sentinel.result3
    assert mock_fn(sentinel.arg1) == sentinel.result3


class TestException(Exception):
    pass


def test_when_call_then_raise():
    mock_fn = Mock()
    when(mock_fn).called_with().then(TestException(sentinel.exception0))
    when(mock_fn).called_with(sentinel.arg1).then(TestException(sentinel.exception1))
    when(mock_fn).called_with(sentinel.arg2).then(TestException(sentinel.exception2))
    when(mock_fn).called_with(sentinel.arg1, sentinel.arg2).then(TestException(sentinel.exception3))
    when(mock_fn).called_with(sentinel.arg1, sentinel.arg1).then(TestException(sentinel.exception4))
    when(mock_fn).called_with(sentinel.arg1, other=sentinel.other).then(TestException(sentinel.exception5))
    when(mock_fn).called_with(x=sentinel.x, y=sentinel.y).then(TestException(sentinel.exception6))

    with pytest.raises(TestException) as err:
        mock_fn()
    assert str(err.value) == str(sentinel.exception0)

    with pytest.raises(TestException) as err:
        mock_fn(sentinel.arg1)
    assert str(err.value) == str(sentinel.exception1)

    with pytest.raises(TestException) as err:
        mock_fn(sentinel.arg2)
    assert str(err.value) == str(sentinel.exception2)

    with pytest.raises(TestException) as err:
        mock_fn(sentinel.arg1, sentinel.arg2)
    assert str(err.value) == str(sentinel.exception3)

    with pytest.raises(TestException) as err:
        mock_fn(sentinel.arg1, sentinel.arg1)
    assert str(err.value) == str(sentinel.exception4)

    with pytest.raises(TestException) as err:
        mock_fn(sentinel.arg1, other=sentinel.other)
    assert str(err.value) == str(sentinel.exception5)

    with pytest.raises(TestException) as err:
        mock_fn(x=sentinel.x, y=sentinel.y)
    assert str(err.value) == str(sentinel.exception6)


def test_when_call_then_raise_single():
    mock_fn = Mock()
    when(mock_fn).called_with(sentinel.arg1).then(TestException(sentinel.exception1))

    with pytest.raises(TestException) as err:
        mock_fn(sentinel.arg1)
    assert str(err.value) == str(sentinel.exception1)

    with pytest.raises(TestException):
        mock_fn(sentinel.arg1)


def test_when_call_then_raise_multiple():
    mock_fn = Mock()
    when(mock_fn).called_with(sentinel.arg1).then(TestException(sentinel.exception1))\
                                            .then(TestException)\
                                            .then(TestException(sentinel.exception3))

    with pytest.raises(TestException) as err:
        mock_fn(sentinel.arg1)
    assert str(err.value) == str(sentinel.exception1)

    with pytest.raises(TestException) as err:
        mock_fn(sentinel.arg1)
    assert str(err.value) == ""

    with pytest.raises(TestException) as err:
        mock_fn(sentinel.arg1)
    assert str(err.value) == str(sentinel.exception3)

    with pytest.raises(TestException):
        mock_fn(sentinel.arg1)


def test_when_call_then_mixed():
    mock_fn = Mock()
    when(mock_fn).called_with(sentinel.arg1).then(sentinel.result1)\
                                            .then(TestException(sentinel.exception2))\
                                            .then(TestException)\
                                            .then(sentinel.result3)

    assert mock_fn(sentinel.arg1) == sentinel.result1

    with pytest.raises(TestException):
        mock_fn(sentinel.arg1)

    with pytest.raises(TestException):
        mock_fn(sentinel.arg1)

    assert mock_fn(sentinel.arg1) == sentinel.result3
    assert mock_fn(sentinel.arg1) == sentinel.result3


def test_when_missing_case():
    mock_fn = Mock()
    when(mock_fn).called_with(sentinel.arg1).then(sentinel.result1)
    when(mock_fn).called_with(sentinel.arg2, sentinel.arg3).then(sentinel.result2)

    with pytest.raises(UnexpectedStubCall) as err:
        mock_fn(sentinel.arg4)
        
    assert str(err.value) == """Unexpected stub call:
    call(sentinel.arg4)
The following calls are configured:
    call(sentinel.arg1)
    call(sentinel.arg2, sentinel.arg3)
"""


@pytest.mark.skipif(sys.version_info < (3, 0), reason="Skip if Python 2")
def test_when_missing_unicode_case_py3():
    mock_fn = Mock()
    when(mock_fn).called_with("hello \u2698").then(sentinel.result1)

    with pytest.raises(UnexpectedStubCall) as err:
        mock_fn("goodbye \u2698")
        
    assert str(err.value) == """Unexpected stub call:
    call('goodbye \u2698')
The following calls are configured:
    call('hello \u2698')
"""


@pytest.mark.skipif(sys.version_info >= (3, 0), reason="Skip if Python 3")
def test_when_missing_unicode_case_py2():
    mock_fn = Mock()
    when(mock_fn).called_with(u"hello \u2698").then(sentinel.result1)

    with pytest.raises(UnexpectedStubCall) as err:
        mock_fn(u"goodbye \u2698")
        
    assert str(err.value) == """Unexpected stub call:
    call(u'goodbye \\u2698')
The following calls are configured:
    call(u'hello \\u2698')
"""


def test_duplicate_called_with_statements_second_ignored():
    mock = Mock()
    when(mock).called_with(100, 200).then("monkey")
    when(mock).called_with(100, 200).then("hello")

    assert 'monkey' == mock(100, 200)
    assert 'hello' != mock(100, 200)


def test_most_general_last():
    mock = Mock()
    when(mock).called_with(100, 200).then("monkey")
    when(mock).called_with(100, Any()).then("hello")

    assert 'monkey' == mock(100, 200)
    assert 'hello' == mock(100, 300)
    assert 'hello' == mock(100, "monkey")
    assert 'hello' == mock(100, { "key" : 1000 })


def test_called_with_object_has_empty_string_representation():
    mock_fn = MagicMock()
    assert repr(when(mock_fn).called_with(sentinel.arg)) == ""



# mockextras.matchers
# Matchers and Stubs for mock.
# Copyright (C) 2012-2015 Man AHL
# E-mail: ManAHLTech AT ahl DOT com

# mockextras 1.0.0
# https://github.com/manahl/mockextras

# Released subject to the BSD License
# Please see https://github.com/manahl/mockextras/blob/master/LICENSE.txt

__all__ = ['Any', 'Contains', 'AnyOf']


class Any(object):
    """Matchers act as wildcards when defining a stub or when asserting call arguments.
    
    The Any matcher will match any object. 
    
    >>> whatever = Any()
    >>> assert whatever == 'hello'
    >>> assert whatever ==  100
    >>> assert whatever ==  range(10)
    
    You can optionally specify a type so that Any only matches objects of that type.
    
    >>> anystring = Any(basestring)
    >>> assert anystring == 'hello'
    >>> assert anystring == 'monkey'
    >>> assert anystring == u'bonjour'
    >>> assert anystring != ['hello', 'world']
    
    Any can be used when specifying stubs:
    
    >>> try:
    ...     from unittest.mock Mock, call
    ... except ImportError:
    ...     from mock import Mock, call
    >>>
    >>> from mockextras import stub
    >>> mock = Mock()
    >>> mock.side_effect = stub((call("hello", "world"), 100),
    ...                         (call("bye bye", Any()), 200))
    >>> mock("bye bye", "world")
    200
    >>> mock("bye bye", "Fred")
    200
    >>> mock("bye bye", range(100))
    200
    >>> mock("bye bye", { 'a' : 1000, 'b' : 2000})
    200
    
    or when asserting call arguments:
    
    >>> try:
    ...     from unittest.mock Mock
    ... except ImportError:
    ...     from mock import Mock
    >>>
    >>> mock = Mock()
    >>> mock("bye bye", "world")
    <Mock name='mock()' id='...'>
    >>> mock.assert_called_once_with("bye bye", Any())
    
    >>> mock("bye bye", "Fred")
    <Mock name='mock()' id='...'>
    >>> assert mock.call_args_list == [call("bye bye", "world"),
    ...                                call("bye bye", Any())]
    """
    def __init__(self, cls=object):
        self._cls = cls

    def __eq__(self, other):
        return isinstance(other, self._cls)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return 'Any(%s)' % ('' if self._cls is object else self._cls)


class Contains(object):
    """Matchers act as wildcards when defining a stub or when asserting call arguments.
    
    The Contains matcher will match objects that contain the given value or substring.
    
    >>> contains_five = Contains(5)
    >>> assert contains_five == range(10)
    >>> assert contains_five != range(4)

    >>> contains_ello = Contains('ello')
    >>> assert contains_ello == "hello"
    >>> assert contains_ello != "bye bye"

    Contains can be used when specifying stubs:
    
    >>> try:
    ...     from unittest.mock Mock, call
    ... except ImportError:
    ...     from mock import Mock, call
    >>>
    >>> from mockextras import stub
    >>> mock = Mock()
    >>> mock.side_effect = stub((call("hello", "world"), 100),
    ...                         (call("bye bye", Contains('monkey')), 200))
    >>> mock("bye bye", "uncle monkey")
    200
    
    or when asserting call arguments:
    
    >>> try:
    ...     from unittest.mock Mock
    ... except ImportError:
    ...     from mock import Mock
    >>>
    >>> mock = Mock()
    >>> mock("bye bye", "world")
    <Mock name='mock()' id='...'>
    >>> mock.assert_called_once_with("bye bye", Contains('or'))
    
    >>> mock("bye bye", "Fred")
    <Mock name='mock()' id='...'>
    >>> assert mock.call_args_list == [call("bye bye", "world"),
    ...                                call("bye bye", Contains('red'))]
    """
    def __init__(self, value):
        self._value = value

    def __eq__(self, other):
        return self._value in other

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return 'Contains(%r)' % self._value


class AnyOf(object):
    """Matchers act as wildcards when defining a stub or when asserting call arguments.
    
    The AnyOf matcher will ....
    
    >>> is_a_small_prime = AnyOf(2,3,5,7,11,13)
    >>> assert is_a_small_prime == 3
    >>> assert is_a_small_prime != 4

    AnyOf can be used when specifying stubs:
    
    >>> try:
    ...     from unittest.mock Mock, call
    ... except ImportError:
    ...     from mock import Mock, call
    >>>
    >>> from mockextras import stub
    >>> mock = Mock()
    >>> mock.side_effect = stub((call("hello"), 100),
    ...                         (call(AnyOf('monkey', 'donkey', 'badger')), 200))
    >>> mock("monkey")
    200
    
    or when asserting call arguments:
    
    >>> try:
    ...     from unittest.mock Mock
    ... except ImportError:
    ...     from mock import Mock
    >>>
    >>> mock = Mock()
    >>> mock("donkey")
    <Mock name='mock()' id='...'>
    >>> mock.assert_called_once_with(AnyOf('monkey', 'donkey', 'badger'))
    
    >>> mock("monkey")
    <Mock name='mock()' id='...'>
    >>> assert mock.call_args_list == [call("donkey"),
    ...                                call(AnyOf('monkey', 'donkey', 'badger'))]
    """
    def __init__(self, *args):
        self._set = set(args)

    def __eq__(self, other):
        return other in self._set

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return 'AnyOf(%s)' % ', '.join(map(repr, self._set))



# mockextras.fluent
# Matchers and Stubs for mock.
# Copyright (C) 2012-2015 Man AHL
# E-mail: ManAHLTech AT ahl DOT com

# mockextras 1.0.0
# https://github.com/manahl/mockextras

# Released subject to the BSD License
# Please see https://github.com/manahl/mockextras/blob/master/LICENSE.txt

from ._matchers import __all__ as matchers_all
from ._stub import _Sequence, _Stub
try:
    from unittest.mock import call, _is_exception, _is_instance_mock
except ImportError:
    try:
        from mock.mock import call, _is_exception, _is_instance_mock
    except ImportError:
        from mock import call, _is_exception, _is_instance_mock
from os import linesep


__all__ = ['when']


def when(mock_fn):
    class ListSeq(_Sequence):
        def __init__(self):
            self.list = []

        def __call__(self):
            if len(self.list) > 1:
                retval = self.list.pop(0)
            else:
                retval = self.list[0]
            if _is_exception(retval):
                raise retval
            return retval

    class When(object):
        def __init__(self, mock_fn):
            self._mock_fn = mock_fn
            if not _is_instance_mock(self._mock_fn):
                raise RuntimeError("mock_fn must be an instance of Mock")

            if not isinstance(self._mock_fn.side_effect, _Stub):
                if self._mock_fn.side_effect is not None:
                    raise RuntimeError("Mock '%s' already has a side_effect set defined" % self._mock_fn)

                self._mock_fn.side_effect = _Stub()

            if not isinstance(self._mock_fn.side_effect._results, list):  #pylint: disable=protected-access
                self._mock_fn.side_effect._results = list(self._mock_fn.side_effect._results)  #pylint: disable=protected-access

        def called_with(self, *args, **kwargs):
            return CalledWith(self._mock_fn.side_effect._results, call(*args, **kwargs))  #pylint: disable=protected-access

    class CalledWith(object):
        def __init__(self, results, key):
            self._results = results
            self._key = key
            self._list = None

        def then(self, obj):
            if self._list is None:
                s = ListSeq()
                self._results.append((self._key, s))
                self._list = s.list
            self._list.append(obj)
            return self

        def __repr__(self):
            return ""

    return When(mock_fn)


when.__doc__ = """Provides a fluent API for specifying stubs.

For example, you can specify different values to return or exceptions to raise based on the arguments
passed into the called_with:

>>> try:
...     from unittest.mock Mock
... except ImportError:
...     from mock import Mock
>>>
>>> mock = Mock()
>>> when(mock).called_with("hello").then("world")
<BLANKLINE>
>>> when(mock).called_with("foo").then("bar")
<BLANKLINE>
>>> when(mock).called_with(100, 200).then(RuntimeError("Boom!"))
<BLANKLINE>
>>> mock("hello")
'world'
>>> mock("foo")
'bar'
>>> mock(100, 200)
Traceback (most recent call last):
...
RuntimeError: Boom!


You can use 'then' multiple times to specify a sequence of results.

>>> mock = Mock()
>>> when(mock).called_with("monkey").then("weezel")\\
...                                 .then("badger")\\
...                                 .then(RuntimeError("Boom!"))
<BLANKLINE>
>>> mock("monkey")
'weezel'
>>> mock("monkey")
'badger'
>>> mock("monkey")
Traceback (most recent call last):
...
RuntimeError: Boom!

The last value is repeated for any subsequent calls.

You can use matchers, such as Any(), as wild-card arguments when matching call arguments. The stub's
configuration is searched in the order it was specified so you can put more specific call argument
specifications ahead of more general ones. For example:

>>> from mockextras import Any
>>> mock = Mock()
>>> when(mock).called_with(100, 200).then("monkey")
<BLANKLINE>
>>> when(mock).called_with(100, Any()).then("hello")
<BLANKLINE>
>>> mock(100, 200)
'monkey'
>>> mock(100, 300)
'hello'
>>> mock(100, "monkey")
'hello'
>>> mock(100, { "key" : 1000 })
'hello'

The following matchers are available in mockextras:
%s

See their documentation for more info.
""" % linesep.join('  * %s' % m for m in matchers_all)



from mockextras import stub, seq, Any, UnexpectedStubCall
from mockextras._stub import _Sequence
try:
    from unittest.mock import Mock, sentinel, patch, call
except ImportError:
    from mock import Mock, sentinel, patch, call
import pytest
from datetime import datetime


def test_seq_empty_list():
    nxt = seq([])
    with pytest.raises(StopIteration):
        nxt()


def test_seq_empty_list_iterator():
    nxt = seq(iter([]))
    with pytest.raises(StopIteration):
        nxt()


def test_seq_empty_generator():
    nxt = seq(i for i in [])
    with pytest.raises(StopIteration):
        nxt()


def test_seq_list():
    nxt = seq([sentinel.val1, sentinel.val2, sentinel.val3])
    assert nxt() == sentinel.val1
    assert nxt() == sentinel.val2
    assert nxt() == sentinel.val3
    with pytest.raises(StopIteration):
        nxt()


def test_seq_list_iterator():
    nxt = seq(iter(iter([sentinel.val1, sentinel.val2, sentinel.val3])))
    assert nxt() == sentinel.val1
    assert nxt() == sentinel.val2
    assert nxt() == sentinel.val3
    with pytest.raises(StopIteration):
        nxt()


def test_seq_generator():
    nxt = seq(i for i in [sentinel.val1, sentinel.val2, sentinel.val3])
    assert nxt() == sentinel.val1
    assert nxt() == sentinel.val2
    assert nxt() == sentinel.val3
    with pytest.raises(StopIteration):
        nxt()


def test_seq_raises_exceptions():
    nxt = seq([sentinel.val1, RuntimeError, RuntimeError(sentinel.val2), sentinel.val3])
    assert nxt() == sentinel.val1

    with pytest.raises(RuntimeError):
        nxt()

    with pytest.raises(RuntimeError) as err:
        nxt()
    assert str(err.value) == str(sentinel.val2)

    assert nxt() == sentinel.val3

    with pytest.raises(StopIteration):
        nxt()


def test_lookup():
    test_data = stub((sentinel.keya, sentinel.vala), (sentinel.keyb, sentinel.valb))

    assert sentinel.vala == test_data._lookup(sentinel.keya)

    assert sentinel.valb == test_data._lookup(sentinel.keyb)


def test_error_on_missed_lookup_on_empty_stub():
    with pytest.raises(UnexpectedStubCall) as err:
        stub()._lookup(sentinel.keya)

    assert str(err.value) == "Unexpected call of an unconfigured stub"


def test_error_on_missed_lookup():
    test_data = stub((sentinel.keya, sentinel.vala), (sentinel.keyb, sentinel.valb))

    with pytest.raises(UnexpectedStubCall) as err:
        test_data._lookup(sentinel.keyc)

    assert str(err.value) == """Unexpected stub call:
    sentinel.keyc
The following calls are configured:
    sentinel.keya
    sentinel.keyb
"""


def test_universal_side_effect():
    st = stub()

    with patch.object(st, "_lookup") as mock_lookup: #@UndefinedVariable
        with patch("mockextras._stub.call") as mock_callargs:
            assert mock_lookup.return_value == st(sentinel.arg1, sentinel.arg2)

    mock_callargs.assert_called_once_with(sentinel.arg1, sentinel.arg2)
    mock_lookup.assert_called_once_with(mock_callargs.return_value)


def test_stub_exception():
    st = stub()

    with patch.object(st, "_lookup", return_value=RuntimeError) as mock_lookup: #@UndefinedVariable
        with patch("mockextras._stub.call") as mock_callargs:
            with pytest.raises(RuntimeError):
                st(sentinel.arg1, sentinel.arg2)

    mock_callargs.assert_called_once_with(sentinel.arg1, sentinel.arg2)
    mock_lookup.assert_called_once_with(mock_callargs.return_value)


def test_stub_sequence():
    st = stub()

    with patch.object(st, "_lookup") as mock_lookup: #@UndefinedVariable
        with patch("mockextras._stub.call") as mock_callargs:
            with patch("mockextras._stub.isinstance", create=True, return_value=True) as mock_isinstance:
                assert mock_lookup.return_value.return_value == st(sentinel.arg1, sentinel.arg2)

    mock_isinstance.assert_called_once_with(mock_lookup.return_value, _Sequence)
    mock_callargs.assert_called_once_with(sentinel.arg1, sentinel.arg2)
    mock_lookup.assert_called_once_with(mock_callargs.return_value)
    mock_lookup.return_value.assert_called_once_with()


def test_stub_missing_case():
    st = stub()

    with patch.object(st, "_lookup", side_effect=UnexpectedStubCall) as mock_lookup: #@UndefinedVariableF
        with patch("mockextras._stub.call") as mock_callargs:
            with pytest.raises(UnexpectedStubCall):
                assert st(sentinel.arg1, sentinel.arg2)

    mock_callargs.assert_called_once_with(sentinel.arg1, sentinel.arg2)
    mock_lookup.assert_called_once_with(mock_callargs.return_value)


def test_stub_switches_on_args():
    mock_fn = Mock()
    mock_fn.side_effect = stub((call(sentinel.argfoo), sentinel.foo),
                               (call(sentinel.whatever, sentinel.argbar), sentinel.bar),
                               (call(sentinel.whatever, sentinel.argbang), RuntimeError),
                               (call(sentinel.whatever, sentinel.argboom), RuntimeError(sentinel.boom)))

    assert mock_fn(sentinel.argfoo) == sentinel.foo

    assert mock_fn(sentinel.whatever, sentinel.argbar) == sentinel.bar

    with pytest.raises(RuntimeError):
        mock_fn(sentinel.whatever, sentinel.argbang)

    with pytest.raises(RuntimeError) as err:
        mock_fn(sentinel.whatever, sentinel.argboom)
    assert str(err.value) == str(sentinel.boom)


def test_stub_sequence_of_results():
    mock_fn = Mock()
    mock_fn.side_effect = stub((call(sentinel.argfoo), sentinel.foo,
                                                       sentinel.bar,
                                                       RuntimeError,
                                                       RuntimeError(sentinel.boom),
                                                       sentinel.all_ok_now))

    assert mock_fn(sentinel.argfoo) == sentinel.foo

    assert mock_fn(sentinel.argfoo) == sentinel.bar

    with pytest.raises(RuntimeError):
        mock_fn(sentinel.argfoo)
        
    with pytest.raises(RuntimeError) as err:
        mock_fn(sentinel.argfoo)
    assert str(err.value) == str(sentinel.boom)

    assert mock_fn(sentinel.argfoo) == sentinel.all_ok_now

    with pytest.raises(StopIteration):
        mock_fn(sentinel.argfoo)


def test_stub_sequence_of_results_from_iterator():
    mock_fn = Mock()
    results = iter([sentinel.foo, sentinel.bar, RuntimeError, RuntimeError(sentinel.boom), sentinel.all_ok_now])
    mock_fn.side_effect = stub((call(sentinel.argfoo), seq(results)))

    assert mock_fn(sentinel.argfoo) == sentinel.foo

    assert mock_fn(sentinel.argfoo) == sentinel.bar

    with pytest.raises(RuntimeError):
        mock_fn(sentinel.argfoo)
        
    with pytest.raises(RuntimeError) as err:
        mock_fn(sentinel.argfoo)
    assert str(err.value) == str(sentinel.boom)

    assert mock_fn(sentinel.argfoo) == sentinel.all_ok_now

    with pytest.raises(StopIteration):
        mock_fn(sentinel.argfoo)


def test_stub_arg_matching():
    mock_fn = Mock()
    mock_fn.side_effect = stub((call(), sentinel.res0),
                               (call(sentinel.arg1), sentinel.res1),
                               (call(datetime(1978, 2, 2, 12, 34, 56)), sentinel.res2),
                               (call(Any()), sentinel.res3),
                               (call(x=sentinel.argx, y=sentinel.argy), sentinel.res4),
                               (call(x=sentinel.argx, y=datetime(1978, 2, 2, 12, 34, 56)), sentinel.res5),
                               (call(x=sentinel.argx, y=Any()), sentinel.res6))

    assert mock_fn() == sentinel.res0
    assert mock_fn(sentinel.arg1) == sentinel.res1
    assert mock_fn(Any()) == sentinel.res1
    assert mock_fn(Any(datetime)) == sentinel.res2
    assert mock_fn(datetime(1978, 2, 2, 12, 34, 56)) == sentinel.res2
    assert mock_fn(datetime(1978, 2, 2, 12, 45, 00)) == sentinel.res3
    assert mock_fn(sentinel.meh) == sentinel.res3
    assert mock_fn(x=sentinel.argx, y=sentinel.argy) == sentinel.res4
    assert mock_fn(x=sentinel.argx, y=datetime(1978, 2, 2, 12, 34, 56)) == sentinel.res5
    assert mock_fn(x=sentinel.argx, y=Any()) == sentinel.res4
    assert mock_fn(x=sentinel.argx, y=Any(datetime)) == sentinel.res5
    assert mock_fn(x=sentinel.argx, y=sentinel.meh) == sentinel.res6



# mockextras.fluent
# Matchers and Stubs for mock.
# Copyright (C) 2012-2015 Man AHL
# E-mail: ManAHLTech AT ahl DOT com

# mockextras 1.0.0
# https://github.com/manahl/mockextras

# Released subject to the BSD License
# Please see https://github.com/manahl/mockextras/blob/master/LICENSE.txt

if __name__ == "__main__":
    params = dict(name="mockextras",
        version="1.0.2",
        description="Extensions to the mock library",
        author="Man AHL",
        author_email="ManAHLTech@ahl.com",
        url="https://github.com/manahl/mockextras",
        packages=['mockextras'],
        license="BSD",
        long_description="""The mockextras library is designed to be used with the unittest.mock library in python 3 or the mock
backport of this (http://www.voidspace.org.uk/python/mock/) in python 2. The mockextras library adds
a number of features that are found in other mocking libraries namely:

* a fluent API for the configuration of stubs
* stubs
* matchers

The documentation is here: http://mockextras.readthedocs.org/
and the source is here: http://github.com/manahl/mockextras""",
        classifiers=["Development Status :: 5 - Production/Stable",
                     "Environment :: Console",
                     "Intended Audience :: Developers",
                     "License :: OSI Approved :: BSD License",
                     "Programming Language :: Python",
                     "Programming Language :: Python :: 2.6",
                     "Programming Language :: Python :: 2.7",
                     "Programming Language :: Python :: 3.3",
                     "Programming Language :: Python :: 3.4",
                     "Programming Language :: Python :: 3.5",
                     "Programming Language :: Python :: Implementation :: CPython",
                     "Programming Language :: Python :: Implementation :: PyPy",
                     "Operating System :: OS Independent",
                     "Topic :: Software Development :: Libraries",
                     "Topic :: Software Development :: Libraries :: Python Modules"])

    try:
        from setuptools import setup
    except ImportError:
        from distutils.core import setup
    else:
        params['install_requires'] = []

    try:
        from unittest import mock
    except ImportError:
        params['install_requires'] += ['mock>=0.8.0']

    setup(**params)
